<?php

namespace Drupal\stackpath\Plugin\Purge\DiagnosticCheck;

use Drupal\purge\Plugin\Purge\DiagnosticCheck\DiagnosticCheckBase;
use Drupal\purge\Plugin\Purge\DiagnosticCheck\DiagnosticCheckInterface;
use Drupal\purge\Plugin\Purge\Purger\PurgersServiceInterface;
use Drupal\stackpath\Entity\StackPathPurgerSettings;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Verifies that only fully configured StackPath purgers load.
 *
 * @PurgeDiagnosticCheck(
 *   id = "purge_purger_stackpath",
 *   title = @Translation("Stackpath"),
 *   description = @Translation("Tests your StackPath configuration."),
 * )
 */
class ConfigurationCheck extends DiagnosticCheckBase implements DiagnosticCheckInterface {

  /**
   * The purgers service.
   *
   * @var \Drupal\purge\Plugin\Purge\Purger\PurgersServiceInterface
   */
  protected $purgePurgers;

  /**
   * Constructs an instance of ConfigurationCheck.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\purge\Plugin\Purge\Purger\PurgersServiceInterface $purge_purgers
   *   The purge executive service, which wipes content from external caches.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PurgersServiceInterface $purge_purgers) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->purgePurgers = $purge_purgers;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('purge.purgers')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function run() {
    // Load configuration objects for all enabled HTTP purgers.
    $plugins = [];
    foreach ($this->purgePurgers->getPluginsEnabled() as $id => $plugin_id) {
      if (in_array($plugin_id, ['purge_purger_stackpath'])) {
        $plugins[$id] = StackPathPurgerSettings::load($id);
      }
    }

    // Perform checks against configuration.
    $labels = $this->purgePurgers->getLabels();
    foreach ($plugins as $id => $settings) {
      $t = ['@purger' => $labels[$id]];
      foreach (['client_id', 'client_secret'] as $f) {
        if (empty($settings->get($f))) {
          $this->recommendation = $this->t("@purger not configured.", $t);
          return self::SEVERITY_ERROR;
        }
      }
    }

    $this->recommendation = "All purgers configured.";
    return self::SEVERITY_OK;
  }

}
